export declare const SEARCH_FLAG = "?";
export declare const BASE_10 = 10;
export type QSOptions = {
    allowEmptySearch?: boolean;
    allowUnknownParams?: boolean;
    allowDefault?: boolean;
    allowUnorderedParams?: boolean;
};
export type QSValue = string | number | boolean;
export type QSParamValue = QSValue;
export type QSParamType = 'string' | 'number' | 'boolean';
export type QSUniqueParameter = {
    name: string;
    description?: string;
    in: 'query';
    pattern?: string;
    required?: boolean;
    enum?: QSParamValue[];
    type: QSParamType;
    default?: QSParamValue;
};
export type QSArrayParameter = {
    name: string;
    description?: string;
    in: 'query';
    type: 'array';
    required?: boolean;
    ordered?: boolean;
    default?: QSParamValue[];
    items: {
        pattern?: string;
        enum?: QSParamValue[];
        type: QSParamType;
    };
};
export type QSParameter = QSUniqueParameter | QSArrayParameter;
export type QSParameterValues = {
    [name: string]: QSValue | QSValue[];
};
/**
 * Parse a queryString according to the provided definitions
 * @param  {Object}  options
 * Parser options
 * @param  {Boolean}  options.allowEmptySearch
 * Avoid throwing when the search is empty
 * @param  {Boolean}  options.allowUnknownParams
 * Avoid throwing when some params are unknown
 * @param  {Boolean}  options.allowDefault
 * Avoid throwing when some params is set to its default value
 * @param  {Boolean}  options.allowUnorderedParams
 * Avoid throwing when params are not set in the same order
 *  than declarations
 * @param  {Array}   definitions
 * Swagger compatible list of defitions
 * @param  {string}  search
 * The actual query string to parse
 * @return {Object}
 * The parsed properties
 * @example
 *
 * import qs from 'strict-qs';
 *
 * const qsOptions = { allowEmptySearch: true };
 * const qsDefinition = [{
 *   name: 'pages',
 *   in: 'query',
 *   type: 'array',
 *   items: {
 *     type: 'number',
 *   },
 *   ordered: true,
 *   description: 'The pages to print',
 *}];
 *
 * qs(qsOptions, qsDefinition, '?pages=0&pages=1&pages=2');
 * // Returns:
 * // {
 * //  pages: [0, 1, 2], // eslint-disable-line
 * // }
 */
declare function qsStrict(options: QSOptions, definitions: QSParameter[], search: string): QSParameterValues;
export declare function parseReentrantNumber(str: string): number;
export declare function parseBoolean(str: string): boolean;
export declare function decodeQueryComponent(value: string): string;
export { qsStrict };
